/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* 2d.c
   Manages 2D actions on game screen */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "2d.h"
#include "world.h"
#include "image.h"
#include "text.h"
#include "cursor.h"

/* size of a tabulation */
#define TTAB 10.

/* external value, managed by 'update' (time related). */
/* this value goes from 0 to 1 and reverse all the time */
extern float blink_value;

/* global transparency */
GLfloat glb_transp=1.;

/* flag for 2D mode */
int flag_2d_mode=0;

/* global pointers */
static World *world=NULL;

/* bitmap for boxes */
#define BOX_W 128
#define BOX_H 128
unsigned char *box_data=NULL;
unsigned char *box_data_sel=NULL;
unsigned char *box_data_usel=NULL;
unsigned char *box_data_down=NULL;

/* convert 0-1 float to 0-255 char */
#define F2C(v) ((unsigned char)((v)*255.))

/* colors for boxes */
GLfloat color_dark[4]={.2, .4, .5, .6};
GLfloat color_medium[4]={.3, .5, .7, .6};
GLfloat color_light[4]={.5, .7, .95, .6};
GLfloat color_inside[4]={.4, .6, .82, .5};
GLfloat color_inside_sel[4]={.45, .65, .9, .5};
GLfloat color_inside_usel[4]={.4, .4, .4, .5};
GLfloat color_inside_down[4]={.2, .4, .5, .5};

/* data for font image */
unsigned char *font_data=NULL;
#define CHAR_W 32
#define CHAR_H 32
#define BCHAR_W 4
#define BCHAR_H 32
#define CHARS_W 1024
#define CHARS_H 512
#define CHAR_WNB 32
#define CHAR_HNB 16
int char_corresp[512]=
{'A','Q','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','R','S','T','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 'U','V','W','X','Y','Z','a','b','c','d','e','f','g','h','i','j','k','l','m','n','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 'o','p','q','r','s','t','u','v','w','x','y','z','0','1','2','3','4','5','6','7','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 '8','9',',',';',':','.','!','?','*','$','=','(',')','/','\\','','-','+','<','>','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 '%','&','"','{','}','[',']','|','`','\'','_','^','@','','','','#','~',256+'!','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 '\0','\0',256+'"',256+'C',256+'R','\0','\0',256+'<',256+'>','\0',256+'+',256+'2',256+'3','','\0',256+'.',256+'1',256+'0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 '\0',256+'?',256+'*',256+'O',256+'/',256+'o','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 256+'U','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 256+'D','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',256+'X',256+'V','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',
 256+'@',' ','\0','\0','\0','\0','\0','\0','\0','\0','\0',256+'m',256+'b',256+'j',256+'e',256+'c',256+'w',256+'s',256+'l',256+'f','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0',};
/* for each char, indicate the real size (from 0. (nothing) to 1. (full) */
float char_sizes[512]={0.};  /* computed by the program */


/* access table */
int char_access[2048]={0};

/* the theoretical ratio screen_x/screen_y */
#define SCREEN_RATIO (640/480)


/* textures corresponding to the boxes and fonts */
GLuint t_box=0;
GLuint t_box_sel=0;
GLuint t_box_usel=0;
GLuint t_down=0;
GLuint t_font=0;




/* init the 2D system */
void init_2d(World *w)
{
  int i, j;

  Nprintf("Initializing 2D module...\n");

  if (w == NULL)
    {
    Neprintf("  Empty world in 2D initialisation.\n");
    Neprintf("Aborted.\n");
    exit(8);
    }
  world = w;
  
  /* Generate the textures */
  regen_boxs(w);
  regen_fonts(w, "images/fonts/font.rgb");

  /* create access table for chars */
  for(i=0; i<2048; i++)
    {
    for(j=0; j<512; j++)
      {
      if (char_corresp[j] == i)
        {
        char_access[i] = j;
        break;
        }
      }
    }

  Nprintf("2D module ready.\n");
}


/* free the 2D module */
void free_2d(World *world)
{
  /* Remove box texture */
  if (t_box != 0)
    glDeleteTextures(1, &t_box);
  /* Remove font texture */
  if (t_font != 0)
    glDeleteTextures(1, &t_font);
}


/* Regenerate the box texture */
void regen_boxs(World *w)
{
  int i, j;

  /* Remove existing texture */
  if (t_box != 0)
    glDeleteTextures(1, &t_box);

  /* Init the bitmaps for boxes */
  if((box_data = malloc(sizeof(unsigned char)*BOX_W*BOX_H*4)) == NULL)
  {
    Neprintf("Not enough memory (init_2d). Aborted.\n");
    exit(9);
  }
  /* Clear */
  for(j=0; j<BOX_W*BOX_H*4; j++)
    box_data[j] = (unsigned char)0;
  /* Create box */
  for(j=2; j<BOX_H-2; j++)
    for(i=2; i<BOX_W-2; i++)
      {
      box_data[(i+j*BOX_W)*4]   = F2C(color_inside[0]);
      box_data[(i+j*BOX_W)*4+1] = F2C(color_inside[1]);
      box_data[(i+j*BOX_W)*4+2] = F2C(color_inside[2]);
      box_data[(i+j*BOX_W)*4+3] = F2C(color_inside[3]);
      }
  for(i=1; i<BOX_W-1; i++)
    {
    box_data[(i+1*BOX_W)*4]   = F2C(color_medium[0]);
    box_data[(i+1*BOX_W)*4+1] = F2C(color_medium[1]);
    box_data[(i+1*BOX_W)*4+2] = F2C(color_medium[2]);
    box_data[(i+1*BOX_W)*4+3] = F2C(color_medium[3]);
    box_data[(i+(BOX_H-2)*BOX_W)*4]   = F2C(color_medium[0]);
    box_data[(i+(BOX_H-2)*BOX_W)*4+1] = F2C(color_medium[1]);
    box_data[(i+(BOX_H-2)*BOX_W)*4+2] = F2C(color_medium[2]);
    box_data[(i+(BOX_H-2)*BOX_W)*4+3] = F2C(color_medium[3]);
    }
  for(i=2; i<BOX_W-2; i++)
    {
    box_data[(i+0*BOX_W)*4]   = F2C(color_light[0]);
    box_data[(i+0*BOX_W)*4+1] = F2C(color_light[1]);
    box_data[(i+0*BOX_W)*4+2] = F2C(color_light[2]);
    box_data[(i+0*BOX_W)*4+3] = F2C(color_light[3]);
    box_data[(i+(BOX_H-1)*BOX_W)*4]   = F2C(color_dark[0]);
    box_data[(i+(BOX_H-1)*BOX_W)*4+1] = F2C(color_dark[1]);
    box_data[(i+(BOX_H-1)*BOX_W)*4+2] = F2C(color_dark[2]);
    box_data[(i+(BOX_H-1)*BOX_W)*4+3] = F2C(color_dark[3]);
    }
  for(j=1; j<BOX_H-1; j++)
    {
    box_data[(1+j*BOX_W)*4]   = F2C(color_medium[0]);
    box_data[(1+j*BOX_W)*4+1] = F2C(color_medium[1]);
    box_data[(1+j*BOX_W)*4+2] = F2C(color_medium[2]);
    box_data[(1+j*BOX_W)*4+3] = F2C(color_medium[3]);
    box_data[(BOX_W-2+j*BOX_W)*4]   = F2C(color_medium[0]);
    box_data[(BOX_W-2+j*BOX_W)*4+1] = F2C(color_medium[1]);
    box_data[(BOX_W-2+j*BOX_W)*4+2] = F2C(color_medium[2]);
    box_data[(BOX_W-2+j*BOX_W)*4+3] = F2C(color_medium[3]);
    }
  for(j=2; j<BOX_H-2; j++)
    {
    box_data[(0+j*BOX_W)*4]   = F2C(color_light[0]);
    box_data[(0+j*BOX_W)*4+1] = F2C(color_light[1]);
    box_data[(0+j*BOX_W)*4+2] = F2C(color_light[2]);
    box_data[(0+j*BOX_W)*4+3] = F2C(color_light[3]);
    box_data[(BOX_W-1+j*BOX_W)*4]   = F2C(color_dark[0]);
    box_data[(BOX_W-1+j*BOX_W)*4+1] = F2C(color_dark[1]);
    box_data[(BOX_W-1+j*BOX_W)*4+2] = F2C(color_dark[2]);
    box_data[(BOX_W-1+j*BOX_W)*4+3] = F2C(color_dark[3]);
    }

  /* Generate the box texture */
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glGenTextures(1, &t_box);
  glBindTexture(GL_TEXTURE_2D, t_box);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, w->texture_mode);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, w->texture_mode);
  glTexImage2D(GL_TEXTURE_2D,
	       0,
	       4,
	       BOX_W,
	       BOX_H,
	       0,
	       GL_RGBA,
	       GL_UNSIGNED_BYTE,
	       (const GLvoid*)box_data);
  free(box_data);
  box_data = NULL;
}

/* Update the properties of the box texture */
void update_boxs(World *w)
{
  glBindTexture(GL_TEXTURE_2D, t_box);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, w->texture_mode);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, w->texture_mode);
}

/* Regenerate the font texture */
void regen_fonts(World *w, const char *fname)
{
  int i, j, tx, ty, k, x, y, xs, jj;

  /* Remove existing texture */
  if (t_font != 0)
    glDeleteTextures(1, &t_font);

  /* prepare font data */
  read_2d_image_rgb((char *)fname, &font_data, &tx, &ty);
  if((tx != CHARS_W)||(ty != CHARS_H))
  {
    Neprintf("Font image is not %dx%d. Aborted.\n", CHARS_W, CHARS_H);
    exit(3);
  }

  /* Compute the X extention for each letter (the last non-transparent
     pixel for each letter area (32x32) */
  k = 0;
  /* For each char */
  for(jj=0; jj<CHAR_HNB; jj++)
    {
    j = 15-jj;
    for(i=0; i<CHAR_WNB; i++)
      {
      /* Check for pixels */
      xs = 0;
      for(x=CHAR_W-1; x>=0; x--)
        {
	for(y=0; y<CHAR_H; y++)
	  {
	  if (font_data[(i*CHAR_W+x+(j*CHAR_WNB*CHAR_W*CHAR_H+y*CHAR_WNB*CHAR_W))*4+3] != 0)
	    {/* Not transparent. It is the letter */
	    xs = x;
	    x = -1;  /* hard exit for loop 1 /o) */
	    break;   /* exit from loop 2 */
	    }
	  }
	}
      if (xs == 0)
        {
	char_sizes[k] = 0.;
	}
      else
      if (xs == CHAR_W-1)
        {
	char_sizes[k] = 1.;
	}
      else
        {
	char_sizes[k] = (float)(xs+1)/CHAR_W;
	}
      k++;
      }
    }
  /* now correct special characters (in particular the SPACE) */
  for(k=0; k<CHAR_HNB*CHAR_WNB; k++)
    {
    if (char_corresp[k] == ' ')
      {
      char_sizes[k] = .3;  /* SPACE is empty but correspond */
      /* to a real extention */
      }
    }

  /* Create the font textures */
  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
  glGenTextures(1, &t_font);
  glBindTexture(GL_TEXTURE_2D, t_font);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, w->texture_mode);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, w->texture_mode);
  glTexImage2D(GL_TEXTURE_2D,
	       0,
	       4,
	       CHARS_W,
	       CHARS_H,
	       0,
	       GL_RGBA,
	       GL_UNSIGNED_BYTE,
	       (const GLvoid*)font_data);
  free(font_data);
  font_data = NULL;
  if ((t_box == 0)||(t_font == 0))
  {
    Neprintf("Error: Can't create GL texture for boxes or fonts. Aborted.\n");
    exit(4);
  }  
}

/* Update the properties of the font texture */
void update_fonts(World *w)
{
  glBindTexture(GL_TEXTURE_2D, t_font);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, w->texture_mode);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, w->texture_mode);
}

/* switch to 2D mode ((0,0)-(100,100)) */
void activate_2d()
{
  /* add our projection matrix (ortho) */
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glOrtho(0, 100, 0, 100, 0, 1);

  /* our own attribs */
  glPushAttrib(GL_COLOR_BUFFER_BIT | GL_LIGHTING_BIT | GL_DEPTH_BUFFER_BIT);

  /* no depth buffer */
  glClear(GL_DEPTH_BUFFER_BIT);
  glDisable(GL_DEPTH_TEST);

  /* no transform */
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  /* no light */
  glDisable(GL_LIGHTING);
  /* transparency properties */
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  /* ready for 2D */
  flag_2d_mode++;
}


/* return to 3D mode */
void deactivate_2d()
{
  /* not in 2D mode */
  if (flag_2d_mode <= 0)
  {
    flag_2d_mode = 0;
    return;
  }
  /* pop 2D stacks */
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glPopAttrib();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  /* remove 2D status */
  flag_2d_mode--;
}


/* draw a 2D box from x,y, with w,h for size */
/* states are: 0: normal,  1: selected, 2: non selectable, 3: pressed */
void draw_2d_box(float x, float y, float w, float h, int state)
{
  glBindTexture(GL_TEXTURE_2D, t_box);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glColor4f(1,1,1,1);
  glEnable(GL_TEXTURE_2D);
  glBegin(GL_QUADS);
  glTexCoord2f(0,0);
  glVertex2f(x, y);
  glTexCoord2f(1,0);
  glVertex2f(x+w, y);
  glTexCoord2f(1,1);
  glVertex2f(x+w, y+h);
  glTexCoord2f(0,1);
  glVertex2f(x, y+h);
  glEnd();
  glDisable(GL_TEXTURE_2D);
}



/* draw a letter with its index */
void draw_2d_letter(int index, float x, float y, float w, float h, int sel)
{
  int i, j;

  j = index / CHAR_WNB;
  j = 15-j;
  i = index % CHAR_WNB;
  glEnable(GL_TEXTURE_2D);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  glBindTexture(GL_TEXTURE_2D, t_font);

  if (sel == 1)
    glColor4f(1., blink_value, blink_value, 1.);
  else
  if(sel == 0)
    glColor4f(1.0, 1.0, 1.0, glb_transp);
  else if(sel == 2)
    glColor4f(1.0, 0.0, 0.0, glb_transp); /* constant red */
  else if(sel == 3)
    glColor4f(0.0, 1.0, 0.0, glb_transp); /* constant green */
  else if(sel == 4)
    glColor4f(0.0, 0.0, 1.0, glb_transp); /* constant blue */

  glBegin(GL_QUADS);
  glTexCoord2f((float)i/(float)CHAR_WNB+(1./CHARS_W), (float)j/(float)CHAR_HNB);
  glVertex2f(x,y);
  glTexCoord2f((float)(i+1)/(float)CHAR_WNB-(1./CHARS_W), (float)(j)/(float)CHAR_HNB);
  glVertex2f(x+w-100./world->size_x,y);
  glTexCoord2f((float)(i+1)/(float)CHAR_WNB-(1./CHARS_W), (float)(j+1)/(float)CHAR_HNB-(1./CHARS_H));
  glVertex2f(x+w-100./world->size_x,y+h-100./world->size_y);
  glTexCoord2f((float)(i)/(float)CHAR_WNB+(1./CHARS_W), (float)(j+1)/(float)CHAR_HNB-(1./CHARS_H));
  glVertex2f(x,y+h-100./world->size_y);
  glEnd();
  glDisable(GL_TEXTURE_2D);
}


/* find letter index */
inline int get_letter_index_simple(int txt)
{
  return(char_access[txt]);
}


/* takes the address of the char because it is planed
   to extend the text with control sequences, in order to
   access to additionnal characters or functions */
/* shift is the number of control characters read */
int get_letter_index(char *txt, int *shift)
{
  *shift = 0;

  /* special: tabular */
  if (txt[0] == '\t')
    return(-1);

  /* extended char */
  if (txt[0] == '#')
    {
    /* '##' is a '#' */
    if (txt[1] == '#')
      {
      *shift = 1;
      return(get_letter_index_simple('#'));
      }
    else
    if ((txt[1] == '\'')||(txt[1] == '`')||(txt[1] == '~')||(txt[1] == '^'))
      {
      *shift = -1;
      return(get_letter_index_simple(txt[1]));
      }
    else
    if (txt[1] == '%')
      {
      *shift = -1;
      return(get_letter_index_simple('"'));
      }
    *shift = 1;
    return(get_letter_index_simple(256+txt[1]));
    }
  else/* standard char */
  return(get_letter_index_simple((int)txt[0]));
}


/* return the size of a letter (in 0-100 units) */
inline float get_2d_letter_size(int size)
{
  return((5 + .5*size));
}

/* return the height of a letter (in 0-100 units) */
inline float get_2d_letter_height(int size)
{
  return(get_2d_letter_size(size)-100./world->size_y);
}

/* return the extention of a letter (in 0-100 units) */
inline float get_2d_letter_extention(int index, int size)
{
  if (index == -1)
    return(1.);
  return((5 + .5*size)*char_sizes[index]);
}

/* return the size of a text (in 0-100 units) */
float get_2d_text_size(char *text, int size)
{
  int i, shift, idx;
  float ts=0.;

  for(i=0; i<strlen(text); i++)
    {
    idx = get_letter_index(&(text[i]), &shift);
    if (idx == -1)
      {/* tabulation! no way to compute exactly the size */
      ts += TTAB/2.;  /* an approximation, because
        the real size depend of the x position (due to
	tabulations) */
      }
    ts += get_2d_letter_extention(idx, size);
    /* accent: dont count the extention */
    if (shift < 0)
      {
      ts -= get_2d_letter_extention(idx, size);
      shift = 1;
      }
    i += shift;
    }
  return(ts);
}

/* find the next tab position */
void go_to_next_tab(float *x)
{
  float p;

  for(p=0.; p<=100.; p+=TTAB)
    {
    if ((*x < p)&&(*x >= p - TTAB))
      {
      *x = p;
      return;
      }
    }
  *x = 100. - TTAB;
}


/* draw a text on the screen */
float get_2d_text_size_full(char *text, int size, float x)
{
  int i, shift, idx;
  float tx, lastx, futx, w, initx;

  initx = x;
  lastx = x;
  futx = x;
  tx = get_2d_letter_size(size);
  for(i=0; i<strlen(text); i++)
    {
    idx = get_letter_index(&(text[i]), &shift);
    if (shift < 0)
      {
      futx = x;
      w = x-lastx;
      x = lastx;
      }
    else
      {
      w = 0;
      }
    if (shift < 0)
      {
      x = futx;
      lastx = x;
      shift = 1;
      }
    else
      {
      lastx = x;
      x += get_2d_letter_extention(idx, size);
      }
    if (idx == -1)
      {/* manage tabulation */
      go_to_next_tab(&x);
      }
    i += shift;
    }
  return(x-initx);
}

/* The dimensions of the previous text drawn */
static float old_text_x = 0.0, old_text_y = 0.0;
static float old_text_width = 0.0, old_text_height = 0.0;

/* draw a text on the screen */
float draw_2d_text(char *text, float x, float y, int size, int sel)
{
  int i, shift, idx;
  float tx, ty, lastx, futx, w, initx;

  old_text_x = x;
  old_text_y = y;
  old_text_height = get_2d_letter_height(size);

  initx = x;
  lastx = x;
  futx = x;
  tx = get_2d_letter_size(size);
  for(i=0; i<strlen(text); i++)
    {
    ty = tx;  /* to correct: respect the screen ratio */
    idx = get_letter_index(&(text[i]), &shift);
    if (shift < 0)
      {
      futx = x;
      w = x-lastx;
      x = lastx;
      draw_2d_letter(idx, x+(w-get_2d_letter_extention(idx, size))/2., y, tx, ty, sel);
      }
    else
      {
      w = 0;
      draw_2d_letter(idx, x, y, tx, ty, sel);
      }
    if (shift < 0)
      {
      x = futx;
      lastx = x;
      shift = 1;
      }
    else
      {
      lastx = x;
      x += get_2d_letter_extention(idx, size);
      }
    if (idx == -1)
      {/* manage tabulation */
      go_to_next_tab(&x);
      }
    i += shift;
    }

  old_text_width = x - initx;

  return old_text_width;
}

/* TRUE if mouse cursor was in the last text drawn */
int cursor_inside_text(void)
{
  float x, y;

  x = get_cursor_x();
  y = get_cursor_y();

  /* Check if in boundaries */
  if((y > old_text_y) && (y < old_text_y + old_text_height) &&
     (x > old_text_x) && (x < old_text_x + old_text_width))
    return TRUE;
  else
    return FALSE;
}

/* TRUE if coords given are inside the text */
int is_inside_text(char *text, float x, float y, int size, float px, float py)
{
  float dx, dy;
  
  /* horizontal check */
  dx = get_2d_text_size_full(text, size, x);
  if ((px < x)||(px > x+dx))
    return(FALSE);
  /* vertical check */
  dy = get_2d_letter_height(size);
  if ((py < y)||(py > y+dy))
    return(FALSE);
  /* inside */
  return(TRUE);
}

/* TRUE if coords given are inside the text (centered) */
int is_inside_text_center(char *text, float x, float w, float y, int size, float px, float py)
{
  float tt;

  tt = get_2d_text_size_full(text, size, x);

  return(is_inside_text(text, x+(w-tt)/2., y, size, px, py));
}

/* TRUE if coords given are inside the text (right justified) */
int is_inside_text_right(char *text, float x, float w, float y, int size, float px, float py)
{
  float tt;

  tt = get_2d_text_size_full(text, size, x);

  return(is_inside_text(text, x+w-tt, y, size, px, py));
}

/* draw a text on the screen, centered betwwen (x) and (x+w) */
float draw_2d_text_center(char *text, float x, float w, float y, int size, int sel)
{
  float tt;

  tt = get_2d_text_size(text, size);

 
  return(draw_2d_text(text, x+(w-tt)/2., y, size, sel));
}

/* draw a text on the screen, right-aligned betwwen (x) and (x+w) */
float draw_2d_text_right(char *text, float x, float w, float y, int size, int sel)
{
  float tt;

  tt = get_2d_text_size(text, size);
  
  return(draw_2d_text(text, x+w-tt, y, size, sel));
}

void set_2d_transparency(GLfloat transp)
{
  glb_transp = transp;
}
