/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __camera_h__
#define __camera_h__

typedef struct _Camera Camera;

#include "world.h"
#include "game.h"
#include "round.h"

/* A camera */
struct _Camera
{
  World *world;
  Game *game;
  Round *round;

  float *target_pos, actual_target_pos[3];
  float pos[3];
  int underwater;

  float rot[3], alt, dist;
  int fixed_alt;

  /* From explosions */
  float shudder_percent;
  float shudder;
  float shudder_vector[3];
  int shudder_vector_valid;
  float shudder_pos[3];
};

/* Add a camera into a round */
Camera *add_camera(Round *round);

void set_camera_target(Camera *, float *);
void move_camera(Camera *, float, float, float);
void move_camerav(Camera *, float *);
void rotate_camera(Camera *, float, float, float);
void zoom_camera(Camera *, float);

void update_cameras(Round *, float);
void update_camera(Camera *, float);
void update_camera_pos(Camera *);

/* Remove a camera */
int remove_camera(Camera *camera);

#endif /* __camera_h__ */
