/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __display_stack_h__
#define __display_stack_h__

typedef enum   _NatowDisplayMode NatowDisplayMode;
typedef struct _DisplayStack DisplayStack;

#include "world.h"

/* The current display mode,
   i.e. what is currently being displayed,
   e.g. a menu, the game etc */
enum _NatowDisplayMode
{
  NATOW_DISPLAY_UNKNOWN = 0,
  NATOW_DISPLAY_MAIN_MENU,
  NATOW_DISPLAY_ABOUT,
  NATOW_DISPLAY_KEY_CONFIG,
  NATOW_DISPLAY_VIDEO_CONFIG,
  NATOW_DISPLAY_GAME_CONFIG,
  NATOW_DISPLAY_PLAYER_CONFIG,
  NATOW_DISPLAY_PLAYER_SHOP,
  NATOW_DISPLAY_START_ROUND,
  NATOW_DISPLAY_ROUND,
  NATOW_DISPLAY_ROUND_MENU,
  NATOW_DISPLAY_ROUND_PAUSED,
  NATOW_DISPLAY_END_ROUND,
  NATOW_DISPLAY_END_GAME,
};

/* A stack of display modes */
struct _DisplayStack
{
  NatowDisplayMode mode;
  DisplayStack *next;
};

/* Set the scene/view/mode to display */
void set_display_mode(World *world, NatowDisplayMode mode);

/* Get the scene/view/mode to display */
NatowDisplayMode get_display_mode(World *world);

/* Push the display mode stack */
void push_display_mode(World *world, NatowDisplayMode mode);

/* Pop the display mode stack */
void pop_display_mode(World *world);

/* Free the natow display stack */
void clear_display_stack(World *world);

/* check if changing for this new screen needs to activate shading */
void check_shade_action(World *world, NatowDisplayMode mode);

/* returns TRUE in going from 'old' to 'new' implies shading */
int check_for_shade(NatowDisplayMode old, NatowDisplayMode new);

#endif /* __display_stack_h__ */
