/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __light_h__
#define __light_h__

#include "map.h"

/* Initialize the precalculated lighting */
int init_lighting(Map *map);

/* Calculates the effect of the sun on the ground */
void calc_map_lighting(Map *map);

/* Shade the whole map */
void shade_map(Map *map);

/* Shades an area of the world map */
void shade_map_area(Map *map,
		    int x0, int y0, int x1, int y1,
		    float *light_dir,
		    float *light_diffuse, float *light_ambient);

/* Same as above but use the worlds sun */
void shade_map_area_with_sun(Map *map, int x0, int y0, int x1, int y1);

/* Add a light into the dynamic light map */
void add_dynamic_light(Map *map, float *pos, float *col, float intensity);

/* Clear all the dynamic lighting */
void clear_dynamic_light_map(Map *map);

/* Clear the area affected by this dynamic light */
void clear_dynamic_light(Map *map, float *pos, float intensity);

#endif /* __light_h__ */
