/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __munition_h__
#define __munition_h__

typedef enum   _MunitionActionType MunitionActionType;
typedef enum   _ActionParameterType ActionParameterType;
typedef union  _ActionParameter ActionParameter;
typedef struct _MunitionAction MunitionAction;
typedef enum   _MunitionType MunitionType;
typedef struct _Munition Munition;

enum _MunitionActionType
{
  MUNITION_UNKNOWN_ACTION = 0,
  MUNITION_CHANGE_TYPE,

  MUNITION_DEPLOY_PARACHUTE,
  MUNITION_DROP_PARACHUTE,

  MUNITION_SET_STATE_RISING,
  MUNITION_SET_STATE_FALLING,
  MUNITION_SET_THRUST,
  MUNITION_SET_DRAG,
  MUNITION_SET_TIMER,
  MUNITION_RESET_TIMER,

  MUNITION_SPAWN_NEW,
  MUNITION_SPAWN_NEW_CLONE,
  MUNITION_SPAWN_NEW_MULTIPLE,
  MUNITION_SPAWN_NEW_CLONE_MULTIPLE,
  MUNITION_SPAWN_NEW_RANDOM,
  MUNITION_SPAWN_NEW_CLONE_RANDOM,
  MUNITION_SPAWN_NEW_RANDOM_MULTIPLE,
  MUNITION_SPAWN_NEW_CLONE_RANDOM_MULTIPLE,
  MUNITION_SPAWN_NEW_SPREAD,
  MUNITION_SPAWN_NEW_CLONE_SPREAD,

  MUNITION_CLIP_TO_COLLISION,

  MUNITION_BOUNCE,
  MUNITION_EXPLODE,

  MUNITION_DELETE
};

enum _ActionParameterType
{
  PARAM_INT = 1,
  PARAM_FLOAT,
  PARAM_STRING
};

union _ActionParameter
{
  int int_param;
  float float_param;
  char *string_param;
};

struct _MunitionAction
{
  MunitionActionType type;
  ActionParameter *parameters;

  MunitionAction *next;
};

enum _MunitionType
{
  MUNITION_NONE = 0,
  MUNITION_SHELL,
  MUNITION_MISSILE,
  MUNITION_BOMBLET
};

struct _Munition
{
  char *name;
  MunitionType type;
  int cost;

  int usable;

  float critical_time;

  MunitionAction *spawn_actions;
  MunitionAction *timer_actions;
  MunitionAction *apex_actions;
  MunitionAction *collision_actions;

  Munition *last;
  Munition *next;
};

#include "world.h"

/* Get the munition named 'name' */
Munition *find_munition(Munition *munition_list, char *name);

/* Loads the munitions from a file */
Munition *load_munition_list(char *file_name);

/* Loads a list of usable munitions, and enables those munitions */
void load_usable_munitions(char *file_name, World *world);

/* Free the munition list */
int clear_munition_list(World *world);

#endif /* __munition_h__ */
