/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __objects_h__
#define __objects_h__

typedef enum   _ObjectType ObjectType;
typedef enum   _ObjectState ObjectState;
typedef struct _Object Object;

#ifdef USE_GLASS
#include <glass.h>
#endif

#include "game.h"

enum _ObjectType
{
  OBJECT_UNKNOWN = 0,
  OBJECT_TREE_1,
  OBJECT_TREE_2,
  OBJECT_TREE_3,
  OBJECT_TREE_DEAD,
  OBJECT_ROCK_SMALL,
  OBJECT_ROCK_MEDIUM,
  OBJECT_ROCK_BIG,
  OBJECT_HOUSE_SMALL,
  OBJECT_HOUSE_MEDIUM,
  OBJECT_HOUSE_BIG,
  OBJECT_HOUSE_DEAD,
  OBJECT_TANK_DEAD,
  OBJECT_CAR_SMALL,
  OBJECT_CAR_MEDIUM,
  OBJECT_CAR_BIG,
  OBJECT_CAR_DEAD,
};


enum _ObjectState
{
  OBJECT_ALIVE = 0,
  OBJECT_DAMAGED,
  OBJECT_DEAD,
  OBJECT_REMOVED
};


/* generic objects */
struct _Object
{
  /*Game *game;*/

  char *name;  /* object name */

  ObjectType type; /* type of the object */

#ifdef USE_GLASS
  /* The GLASS model for the object */
  GlassObject *model;
  int component;
  /* The GLASS model when object is killed */
  GlassObject *model_dead;
  int component_dead;
#endif

  /* state of the object */
  enum _ObjectState state;

  /* Position */
  float pos[3], vel[3], acc[3];

  /* Time during which object is displayed dead */
  float time_dead;

  /* Health of the object (<0: non destroyable) */
  float health;
  float mass;
  int movable;  /* object is not fixed on the ground */
  int floatable;  /* object can float on water */
  
  /* Links */
  Object *next, *last;
  /* special link */
  Object *next2;
};

/* load an object file */
Object *load_object(Game *game, char *fname);

/* allocate a new object */
Object * allocate_object(Game *game, ObjectType type);

/* delete an object */
void delete_object(Object *obj);

/* add an object in the list */
void add_object(Game *game, Object *obj);

/* remove an object from the list (Note: it freed it) */
void remove_object(Game *game, Object *obj);

/* set object mass */
void set_object_mass(Object *object, float mass);

/* get object mass */
float get_object_mass(Object *object);

/* set object health */
void set_object_health(Object *object, float health);

/* get object health */
float get_object_health(Object *object);

/* set object 'movable' */
void set_object_movable(Object *object, int movable);

/* get object 'movable' */
int get_object_movable(Object *object);

/* set object name */
void set_object_name(Object *object, char *name);

/* get object name */
char *get_object_name(Object *object);

/* give damages to objects in the given list */
void damage_objects(Game *game, Object *objects, float *pos, float radius, float force);

/* add object at given coords */
void add_object_in_map(Game *game, Object *object, int x, int y);

/* update all objects */
void update_objects(Game *game, float dt);

/* update an object */
void update_object(Game *game, Object *object, float dt);

/* draw all objects */
void draw_objects(Game *game);

/* draw an object */
void draw_object(Game *game, Object *object);

#endif /* __objects_h__ */
