/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* text.c
   Contains tools for debug and text output (stdout) */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "text.h"


/* 'quiet' status: do not display messages */
static int text_quiet=0;

/* debug level */
static int text_debug=0;



/* set the quiet mode */
void set_quiet_mode(int mode)
{
  text_quiet = mode;
}


/* set debug level */
void set_debug_level(int level)
{
  if (level < 0)
    text_debug = 0;
  else
    text_debug = level;
}

/* NATOW printf. same than printf but with 'quiet' management */
void Nprintf(char *format, ...)
{
  va_list args;

  if (text_quiet)
    return;

  va_start(args, format);
  fprintf(stdout, "NATOW: ");
  vfprintf(stdout, format, args);
  va_end(args);
}

/* Same as above, but without the NATOW: header */
void Nprintf_simple(char *format, ...)
{
  va_list args;

  if (text_quiet)
    return;

  va_start(args, format);
  vfprintf(stdout, format, args);
  va_end(args);
}

/* NATOW fprintf(stderr...). same than fprintf but with 'quiet' management */
void Neprintf(char *format, ...)
{
  va_list args;

  if (text_quiet)
    return;

  va_start(args, format);
  fprintf(stderr, "NATOW: ");
  vfprintf(stderr, format, args);
  va_end(args);
}
