/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __trails_h__
#define __trails_h__

typedef struct _Trail Trail;

#include "world.h"
#include "game.h"
#include "round.h"

/* A smoke trail from a projectile */
struct _Trail
{
  World *world;
  Game *game;
  Round *round;

  float pos[2][3];
  float up[2][3];
  float left[2][3];
  GLfloat colour[4];
  float decay_factor;
  float decay0, decay1;

  Trail *last, *next;
};

/* Prototypes */
Trail *add_trail(Round *round, float *pos0, float *pos1,
		 float *up0, float *up1,
		 float *left0, float *left1,
		 float *colour, float decay_factor, float dt);
void remove_trail(Trail *);
void update_trails(Round *round, float);
void update_trail(Trail *, float);
void draw_trails(Round *round);
void draw_trail(Trail *);

/* Remove all trails */
int clear_trails(Round *round);

#endif /* __trails_h__ */
