/* 
 * NATOW - Tank to tank warfare in a 3D fully destructable world
 *
 * Copyright (C) 2000-2001
 *                     Robert Ancell  <bob27@users.sourceforge.net>
 *                     Yannick Perret <hexasoft@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __vector_h_
#define __vector_h_

/* Make a 3d vector out of three floats */
float *make_vector(float x, float y, float z);

/* Duplicate a 3d vector */
float *vector_dup(float *v);

/* Normalize a 3d vector (using the 2-norm) */
float *normalize_vector(float *);

/* Return the 1-norm of a 3d vector */
float vector_1norm(float *v);

/* Return the 2-norm of a 3d vector (cartesian length) */
float vector_2norm(float *v);

/* Return the 2-norm of a 3d vector without the 'sqrt' */
float vector_norm(float *v);

/* Return the infinity-norm of a 3d vector */
float vector_infnorm(float *v);

/* Add two 3d vectors */
float *vector_add(float *v1, float *v2);

/* Subtract two 3d vectors (v1 - v2) */
float *vector_sub(float *v1, float *v2);

/* Scale a 3d vector by a constant */
float *vector_scale(float scalar, float *v0);

/* Dot product for two 3d vectors */
float vector_dot(float *v1, float *v2);

/* Cross product */
float *vector_cross(float *v1, float *v2);

/* Makes a normalized vector that points in a random (spherical) direction */
float *make_random_vector(void);

/* Make a random vector that deviates a maximum of 'angle' from the direction */
float *make_random_spread(float *dir, float dangle);

/* Formats a 3d vector an prints it to stdout */
void print_vector(float *v);

/* distance between two points */
float vector_distance(float *v1, float *v2);

/* Same as previous but without allocations */
void _make_vector(float x, float y, float z, float *v);
float *_vector_dup(float *v0, float *v);
void _vector_add(float *v1, float *v2, float *v);
float *_vector_sub(float *v1, float *v2, float *v);
void _vector_scale(float scalar, float *v0, float *v);
void _vector_cross(float *v1, float *v2, float *v);
void _make_random_vector(float *v);
void _make_random_spread(float *dir, float dangle, float *v);

#endif /* __vector_h_ */
